var _DatePickerDialog_isResetAction, _DatePickerDialog_selectedDate, _DatePickerDialog_valueAsDate, _DatePickerDialog_onClosed, _DatePickerDialog_onClosing, _DatePickerDialog_onOpened, _DatePickerDialog_onOpening, _DatePickerDialog_onResetClick;
import { __classPrivateFieldGet, __classPrivateFieldSet, __decorate } from "tslib";
import '@material/mwc-button';
import '@material/mwc-dialog';
import '../date-picker/app-date-picker.js';
import './app-date-picker-dialog-base.js';
import { html, nothing } from 'lit';
import { property, queryAsync, state } from 'lit/decorators.js';
import { appDatePickerName } from '../date-picker/constants.js';
import { slotDatePicker } from '../helpers/slot-date-picker.js';
import { toDateString } from '../helpers/to-date-string.js';
import { toResolvedDate } from '../helpers/to-resolved-date.js';
import { DatePickerMinMaxMixin } from '../mixins/date-picker-min-max-mixin.js';
import { DatePickerMixin } from '../mixins/date-picker-mixin.js';
import { RootElement } from '../root-element/root-element.js';
import { baseStyling } from '../stylings.js';
import { datePickerDialogStyling } from './stylings.js';
export class DatePickerDialog extends DatePickerMixin(DatePickerMinMaxMixin(RootElement)) {
    constructor() {
        super();
        this.confirmLabel = 'set';
        this.dismissLabel = 'cancel';
        this.open = false;
        this.resetLabel = 'reset';
        this._rendered = false;
        _DatePickerDialog_isResetAction.set(this, false);
        _DatePickerDialog_selectedDate.set(this, void 0);
        _DatePickerDialog_valueAsDate.set(this, void 0);
        _DatePickerDialog_onClosed.set(this, async (ev) => {
            const datePicker = await this._datePicker;
            this.hide();
            datePicker && (datePicker.startView = 'calendar');
            this.fire({ detail: ev.detail, type: 'closed' });
        });
        _DatePickerDialog_onClosing.set(this, ({ detail, }) => {
            if (detail.action === 'set') {
                const selectedDate = __classPrivateFieldGet(this, _DatePickerDialog_selectedDate, "f");
                __classPrivateFieldSet(this, _DatePickerDialog_valueAsDate, new Date(selectedDate), "f");
                this.value = toDateString(selectedDate);
            }
            this.fire({ detail, type: 'closing' });
        });
        _DatePickerDialog_onOpened.set(this, ({ detail }) => {
            this.fire({ detail, type: 'opened' });
        });
        _DatePickerDialog_onOpening.set(this, ({ detail }) => {
            this.fire({ detail, type: 'opening' });
        });
        _DatePickerDialog_onResetClick.set(this, () => {
            __classPrivateFieldSet(this, _DatePickerDialog_isResetAction, true, "f");
            this.value = undefined;
        });
        __classPrivateFieldSet(this, _DatePickerDialog_selectedDate, __classPrivateFieldSet(this, _DatePickerDialog_valueAsDate, toResolvedDate(), "f"), "f");
    }
    get valueAsDate() {
        return __classPrivateFieldGet(this, _DatePickerDialog_valueAsDate, "f");
    }
    get valueAsNumber() {
        return +__classPrivateFieldGet(this, _DatePickerDialog_valueAsDate, "f");
    }
    willUpdate(changedProperties) {
        super.willUpdate(changedProperties);
        if (!this._rendered && this.open) {
            this._rendered = true;
        }
    }
    updated(changedProperties) {
        if (changedProperties.has('value')) {
            __classPrivateFieldSet(this, _DatePickerDialog_selectedDate, __classPrivateFieldSet(this, _DatePickerDialog_valueAsDate, toResolvedDate(this.value), "f"), "f");
        }
    }
    render() {
        const { _rendered, confirmLabel, dismissLabel, open, resetLabel, } = this;
        return html `
    <app-date-picker-dialog-base
      ?open=${open}
      @closed=${__classPrivateFieldGet(this, _DatePickerDialog_onClosed, "f")}
      @closing=${__classPrivateFieldGet(this, _DatePickerDialog_onClosing, "f")}
      @date-updated=${this.$onDatePickerDateUpdated}
      @first-updated=${this.$onDatePickerFirstUpdated}
      @opened=${__classPrivateFieldGet(this, _DatePickerDialog_onOpened, "f")}
      @opening=${__classPrivateFieldGet(this, _DatePickerDialog_onOpening, "f")}
    >
      ${_rendered ? html `
      ${this.open ? this.$renderSlot() : nothing}

      <div class=secondary-actions slot=secondaryAction>
        <mwc-button
          @click=${__classPrivateFieldGet(this, _DatePickerDialog_onResetClick, "f")}
          data-dialog-action=reset
        >${resetLabel}</mwc-button>
        <mwc-button
          dialogAction=cancel
        >${dismissLabel}</mwc-button>
      </div>

      <mwc-button
        dialogAction=set
        slot=primaryAction
      >${confirmLabel}</mwc-button>
      ` : nothing}
    </app-date-picker-dialog-base>
    `;
    }
    hide() {
        this.open = false;
    }
    show() {
        this.open = true;
    }
    $onDatePickerDateUpdated({ detail: { valueAsDate, }, }) {
        __classPrivateFieldSet(this, _DatePickerDialog_selectedDate, new Date(valueAsDate), "f");
        if (__classPrivateFieldGet(this, _DatePickerDialog_isResetAction, "f") || !this.value) {
            __classPrivateFieldSet(this, _DatePickerDialog_isResetAction, false, "f");
            __classPrivateFieldSet(this, _DatePickerDialog_valueAsDate, new Date(valueAsDate), "f");
            this.value = toDateString(valueAsDate);
        }
    }
    $onDatePickerFirstUpdated({ detail: { valueAsDate, }, }) {
        __classPrivateFieldSet(this, _DatePickerDialog_selectedDate, __classPrivateFieldSet(this, _DatePickerDialog_valueAsDate, valueAsDate, "f"), "f");
    }
    $renderSlot() {
        const { chooseMonthLabel, chooseYearLabel, disabledDates, disabledDays, firstDayOfWeek, locale, max, min, nextMonthLabel, previousMonthLabel, selectedDateLabel, selectedYearLabel, shortWeekLabel, showWeekNumber, startView, todayLabel, toyearLabel, value, weekLabel, weekNumberTemplate, weekNumberType, } = this;
        return slotDatePicker({
            chooseMonthLabel,
            chooseYearLabel,
            disabledDates,
            disabledDays,
            firstDayOfWeek,
            locale,
            max,
            min,
            nextMonthLabel,
            onDatePickerDateUpdated: this.$onDatePickerDateUpdated,
            onDatePickerFirstUpdated: this.$onDatePickerFirstUpdated,
            previousMonthLabel,
            selectedDateLabel,
            selectedYearLabel,
            shortWeekLabel,
            showWeekNumber,
            startView,
            todayLabel,
            toyearLabel,
            value,
            weekLabel,
            weekNumberTemplate,
            weekNumberType,
        });
    }
}
_DatePickerDialog_isResetAction = new WeakMap(), _DatePickerDialog_selectedDate = new WeakMap(), _DatePickerDialog_valueAsDate = new WeakMap(), _DatePickerDialog_onClosed = new WeakMap(), _DatePickerDialog_onClosing = new WeakMap(), _DatePickerDialog_onOpened = new WeakMap(), _DatePickerDialog_onOpening = new WeakMap(), _DatePickerDialog_onResetClick = new WeakMap();
DatePickerDialog.styles = [
    baseStyling,
    datePickerDialogStyling,
];
__decorate([
    property({ type: String })
], DatePickerDialog.prototype, "confirmLabel", void 0);
__decorate([
    property({ type: String })
], DatePickerDialog.prototype, "dismissLabel", void 0);
__decorate([
    property({ type: Boolean })
], DatePickerDialog.prototype, "open", void 0);
__decorate([
    property({ type: String })
], DatePickerDialog.prototype, "resetLabel", void 0);
__decorate([
    queryAsync(appDatePickerName)
], DatePickerDialog.prototype, "_datePicker", void 0);
__decorate([
    state()
], DatePickerDialog.prototype, "_rendered", void 0);
