var _DatePickerInput_disconnect, _DatePickerInput_focusElement, _DatePickerInput_lazyLoading, _DatePickerInput_picker, _DatePickerInput_valueAsDate, _DatePickerInput_valueFormatter, _DatePickerInput_lazyLoad, _DatePickerInput_onResetClick, _DatePickerInput_onClosed, _DatePickerInput_onDatePickerDateUpdated, _DatePickerInput_onDatePickerFirstUpdated, _DatePickerInput_onOpened, _DatePickerInput_updateValues;
import { __classPrivateFieldGet, __classPrivateFieldSet, __decorate } from "tslib";
import '@material/mwc-textfield';
import '../icon-button/app-icon-button.js';
import { TextField } from '@material/mwc-textfield';
import { nothing } from 'lit';
import { html } from 'lit';
import { property, queryAsync, state } from 'lit/decorators.js';
import { ifDefined } from 'lit/directives/if-defined.js';
import { DateTimeFormat } from '../constants.js';
import { appDatePickerName } from '../date-picker/constants.js';
import { appDatePickerInputSurfaceName } from '../date-picker-input-surface/constants.js';
import { slotDatePicker } from '../helpers/slot-date-picker.js';
import { toDateString } from '../helpers/to-date-string.js';
import { warnUndefinedElement } from '../helpers/warn-undefined-element.js';
import { appIconButtonName } from '../icon-button/constants.js';
import { iconClear } from '../icons.js';
import { keyEnter, keyEscape, keySpace, keyTab } from '../key-values.js';
import { DatePickerMinMaxMixin } from '../mixins/date-picker-min-max-mixin.js';
import { DatePickerMixin } from '../mixins/date-picker-mixin.js';
import { ElementMixin } from '../mixins/element-mixin.js';
import { baseStyling } from '../stylings.js';
import { appDatePickerInputClearLabel, appDatePickerInputType } from './constants.js';
import { datePickerInputStyling } from './stylings.js';
export class DatePickerInput extends ElementMixin(DatePickerMixin(DatePickerMinMaxMixin(TextField))) {
    constructor() {
        super(...arguments);
        this.iconTrailing = 'clear';
        this.type = appDatePickerInputType;
        this.clearLabel = appDatePickerInputClearLabel;
        this._disabled = false;
        this._lazyLoaded = false;
        this._open = false;
        _DatePickerInput_disconnect.set(this, () => undefined);
        _DatePickerInput_focusElement.set(this, undefined);
        _DatePickerInput_lazyLoading.set(this, false);
        _DatePickerInput_picker.set(this, undefined);
        _DatePickerInput_valueAsDate.set(this, void 0);
        _DatePickerInput_valueFormatter.set(this, this.$toValueFormatter());
        _DatePickerInput_lazyLoad.set(this, async () => {
            if (this._lazyLoaded || __classPrivateFieldGet(this, _DatePickerInput_lazyLoading, "f"))
                return;
            const deps = [
                appDatePickerName,
                appDatePickerInputSurfaceName,
            ];
            if (deps.some(n => globalThis.customElements.get(n) == null)) {
                __classPrivateFieldSet(this, _DatePickerInput_lazyLoading, true, "f");
                const tasks = deps.map(n => globalThis.customElements.whenDefined(n));
                const imports = [
                    import('../date-picker/app-date-picker.js'),
                    import('../date-picker-input-surface/app-date-picker-input-surface.js'),
                ];
                try {
                    await Promise.all(imports);
                    await Promise.all(tasks);
                }
                catch (error) {
                    console.error(error);
                }
            }
            await this.updateComplete;
            __classPrivateFieldSet(this, _DatePickerInput_lazyLoading, false, "f");
            this._lazyLoaded = true;
        });
        _DatePickerInput_onResetClick.set(this, (ev) => {
            if (this._disabled)
                return;
            ev.preventDefault();
            this.reset();
        });
        _DatePickerInput_onClosed.set(this, ({ detail }) => {
            this._open = false;
            this.fire({ detail, type: 'closed' });
        });
        _DatePickerInput_onDatePickerDateUpdated.set(this, async (ev) => {
            const { isKeypress, key, valueAsDate, } = ev.detail;
            if (!isKeypress || (key === keyEnter || key === keySpace)) {
                __classPrivateFieldGet(this, _DatePickerInput_updateValues, "f").call(this, valueAsDate);
                isKeypress && (await this.$inputSurface)?.close();
            }
        });
        _DatePickerInput_onDatePickerFirstUpdated.set(this, ({ currentTarget, detail: { focusableElements: [focusableElement], valueAsDate, }, }) => {
            __classPrivateFieldSet(this, _DatePickerInput_focusElement, focusableElement, "f");
            __classPrivateFieldSet(this, _DatePickerInput_picker, currentTarget, "f");
            __classPrivateFieldGet(this, _DatePickerInput_updateValues, "f").call(this, valueAsDate);
        });
        _DatePickerInput_onOpened.set(this, async ({ detail }) => {
            await __classPrivateFieldGet(this, _DatePickerInput_picker, "f")?.updateComplete;
            await this.updateComplete;
            __classPrivateFieldGet(this, _DatePickerInput_focusElement, "f")?.focus();
            this.fire({ detail, type: 'opened' });
        });
        _DatePickerInput_updateValues.set(this, (value) => {
            if (value) {
                const valueDate = new Date(value);
                __classPrivateFieldSet(this, _DatePickerInput_valueAsDate, valueDate, "f");
                this.value = toDateString(valueDate);
            }
            else {
                this.reset();
            }
        });
    }
    get valueAsDate() {
        return __classPrivateFieldGet(this, _DatePickerInput_valueAsDate, "f") || null;
    }
    get valueAsNumber() {
        return Number(__classPrivateFieldGet(this, _DatePickerInput_valueAsDate, "f") || NaN);
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        __classPrivateFieldGet(this, _DatePickerInput_disconnect, "f").call(this);
    }
    async firstUpdated() {
        super.firstUpdated();
        const input = await this.$input;
        if (input) {
            const onBodyKeyup = async (ev) => {
                if (this._disabled)
                    return;
                if (ev.key === keyEscape) {
                    this.closePicker();
                }
                else if (ev.key === keyTab) {
                    const inputSurface = await this.$inputSurface;
                    const isTabInsideInputSurface = ev.composedPath().find(n => n.nodeType === Node.ELEMENT_NODE &&
                        n.isEqualNode(inputSurface));
                    if (!isTabInsideInputSurface)
                        this.closePicker();
                }
            };
            const onClick = () => {
                if (this._disabled)
                    return;
                this._open = true;
            };
            const onKeyup = (ev) => {
                if (this._disabled)
                    return;
                if ([keySpace, keyEnter].some(n => n === ev.key)) {
                    onClick();
                }
            };
            document.body.addEventListener('keyup', onBodyKeyup);
            input.addEventListener('keyup', onKeyup);
            input.addEventListener('click', onClick);
            __classPrivateFieldSet(this, _DatePickerInput_disconnect, () => {
                document.body.removeEventListener('keyup', onBodyKeyup);
                input.removeEventListener('keyup', onKeyup);
                input.removeEventListener('click', onClick);
            }, "f");
        }
    }
    willUpdate(changedProperties) {
        super.willUpdate(changedProperties);
        if (changedProperties.has('locale')) {
            const newLocale = (this.locale || DateTimeFormat().resolvedOptions().locale);
            this.locale = newLocale;
            __classPrivateFieldSet(this, _DatePickerInput_valueFormatter, this.$toValueFormatter(), "f");
            __classPrivateFieldGet(this, _DatePickerInput_updateValues, "f").call(this, this.value);
        }
        if (changedProperties.has('value')) {
            __classPrivateFieldGet(this, _DatePickerInput_updateValues, "f").call(this, this.value);
        }
        if (changedProperties.has('disabled') || changedProperties.has('readOnly')) {
            this._disabled = this.disabled || this.readOnly;
        }
        if (changedProperties.has('disabled') || changedProperties.has('outlined')) {
            this.layout();
        }
    }
    async updated() {
        if (this._open && this._lazyLoaded) {
            const picker = await this.$picker;
            picker?.queryAll?.(appIconButtonName).forEach(n => n.layout());
        }
    }
    render() {
        const { _lazyLoaded, _open, } = this;
        if (!_lazyLoaded && _open)
            __classPrivateFieldGet(this, _DatePickerInput_lazyLoad, "f").call(this);
        return html `
    ${super.render()}
    ${_lazyLoaded ? this.$renderContent() : nothing}
    `;
    }
    closePicker() {
        this._open = false;
    }
    reset() {
        if (this._disabled)
            return;
        __classPrivateFieldSet(this, _DatePickerInput_valueAsDate, undefined, "f");
        this.value = '';
    }
    showPicker() {
        if (this._disabled)
            return;
        this._open = true;
    }
    renderInput(shouldRenderHelperText) {
        const { autocapitalize, disabled, focused, helperPersistent, inputMode, isUiValid, label, name, placeholder, required, validationMessage, } = this;
        const autocapitalizeOrUndef = autocapitalize ?
            autocapitalize :
            undefined;
        const showValidationMessage = validationMessage && !isUiValid;
        const ariaLabelledbyOrUndef = label ? 'label' : undefined;
        const ariaControlsOrUndef = shouldRenderHelperText ? 'helper-text' : undefined;
        const ariaDescribedbyOrUndef = focused || helperPersistent || showValidationMessage ?
            'helper-text' :
            undefined;
        const valueText = __classPrivateFieldGet(this, _DatePickerInput_valueAsDate, "f") ? __classPrivateFieldGet(this, _DatePickerInput_valueFormatter, "f").format(__classPrivateFieldGet(this, _DatePickerInput_valueAsDate, "f")) : '';
        return html `
      <input
          ?disabled=${disabled}
          ?required=${required}
          .value=${valueText}
          @blur=${this.onInputBlur}
          @focus=${this.onInputFocus}
          aria-controls=${ifDefined(ariaControlsOrUndef)}
          aria-describedby=${ifDefined(ariaDescribedbyOrUndef)}
          aria-labelledby=${ifDefined(ariaLabelledbyOrUndef)}
          autocapitalize=${ifDefined(autocapitalizeOrUndef)}
          class=mdc-text-field__input
          inputmode=${ifDefined(inputMode)}
          name=${ifDefined(name || undefined)}
          placeholder=${ifDefined(placeholder)}
          readonly
          type=text
      >`;
    }
    renderTrailingIcon() {
        return html `
    <app-icon-button
      .disabled=${this._disabled}
      @click=${__classPrivateFieldGet(this, _DatePickerInput_onResetClick, "f")}
      aria-label=${this.clearLabel}
      class="mdc-text-field__icon mdc-text-field__icon--trailing"
    >
      ${iconClear}
    </app-icon-button>
    `;
    }
    $renderContent() {
        warnUndefinedElement(appDatePickerInputSurfaceName);
        return html `
    <app-date-picker-input-surface
      @opened=${__classPrivateFieldGet(this, _DatePickerInput_onOpened, "f")}
      ?open=${this._open}
      ?stayOpenOnBodyClick=${true}
      .anchor=${this}
      @closed=${__classPrivateFieldGet(this, _DatePickerInput_onClosed, "f")}
    >${this._open ? this.$renderSlot() : nothing}</app-date-picker-input-surface>
    `;
    }
    $renderSlot() {
        const { chooseMonthLabel, chooseYearLabel, disabledDates, disabledDays, firstDayOfWeek, locale, max, min, nextMonthLabel, previousMonthLabel, selectedDateLabel, selectedYearLabel, shortWeekLabel, showWeekNumber, startView, todayLabel, toyearLabel, value, weekLabel, weekNumberTemplate, weekNumberType, } = this;
        return slotDatePicker({
            chooseMonthLabel,
            chooseYearLabel,
            disabledDates,
            disabledDays,
            firstDayOfWeek,
            locale,
            max,
            min,
            nextMonthLabel,
            onDatePickerDateUpdated: __classPrivateFieldGet(this, _DatePickerInput_onDatePickerDateUpdated, "f"),
            onDatePickerFirstUpdated: __classPrivateFieldGet(this, _DatePickerInput_onDatePickerFirstUpdated, "f"),
            previousMonthLabel,
            selectedDateLabel,
            selectedYearLabel,
            shortWeekLabel,
            showWeekNumber,
            startView,
            todayLabel,
            toyearLabel,
            value,
            weekLabel,
            weekNumberTemplate,
            weekNumberType,
        });
    }
    $toValueFormatter() {
        return DateTimeFormat(this.locale, {
            year: 'numeric',
            month: 'short',
            day: 'numeric',
        });
    }
}
_DatePickerInput_disconnect = new WeakMap(), _DatePickerInput_focusElement = new WeakMap(), _DatePickerInput_lazyLoading = new WeakMap(), _DatePickerInput_picker = new WeakMap(), _DatePickerInput_valueAsDate = new WeakMap(), _DatePickerInput_valueFormatter = new WeakMap(), _DatePickerInput_lazyLoad = new WeakMap(), _DatePickerInput_onResetClick = new WeakMap(), _DatePickerInput_onClosed = new WeakMap(), _DatePickerInput_onDatePickerDateUpdated = new WeakMap(), _DatePickerInput_onDatePickerFirstUpdated = new WeakMap(), _DatePickerInput_onOpened = new WeakMap(), _DatePickerInput_updateValues = new WeakMap();
DatePickerInput.styles = [
    ...TextField.styles,
    baseStyling,
    datePickerInputStyling,
];
__decorate([
    property({ type: String })
], DatePickerInput.prototype, "clearLabel", void 0);
__decorate([
    queryAsync('.mdc-text-field__input')
], DatePickerInput.prototype, "$input", void 0);
__decorate([
    queryAsync(appDatePickerInputSurfaceName)
], DatePickerInput.prototype, "$inputSurface", void 0);
__decorate([
    queryAsync(appDatePickerName)
], DatePickerInput.prototype, "$picker", void 0);
__decorate([
    state()
], DatePickerInput.prototype, "_disabled", void 0);
__decorate([
    state()
], DatePickerInput.prototype, "_lazyLoaded", void 0);
__decorate([
    state()
], DatePickerInput.prototype, "_open", void 0);
