var _DatePicker_formatters, _DatePicker_focusNavButtonWithKey, _DatePicker_today, _DatePicker_valueAsDate, _DatePicker_navigateMonth, _DatePicker_queryAllFocusable, _DatePicker_renderCalendar, _DatePicker_renderNavigationButton, _DatePicker_renderYearGrid, _DatePicker_updateSelectedDate, _DatePicker_updateStartView, _DatePicker_updateYear;
import { __classPrivateFieldGet, __classPrivateFieldSet, __decorate } from "tslib";
import '../icon-button/app-icon-button.js';
import '../month-calendar/app-month-calendar.js';
import '../year-grid/app-year-grid.js';
import { html, nothing } from 'lit';
import { queryAsync, state } from 'lit/decorators.js';
import { classMap } from 'lit/directives/class-map.js';
import { ifDefined } from 'lit/directives/if-defined.js';
import { calendar } from 'nodemod/dist/calendar/calendar.js';
import { getWeekdays } from 'nodemod/dist/calendar/helpers/get-weekdays.js';
import { toUTCDate } from 'nodemod/dist/calendar/helpers/to-utc-date.js';
import { DateTimeFormat, MAX_DATE, startViews } from '../constants.js';
import { clampValue } from '../helpers/clamp-value.js';
import { dateValidator } from '../helpers/date-validator.js';
import { focusElement } from '../helpers/focus-element.js';
import { isInCurrentMonth } from '../helpers/is-in-current-month.js';
import { splitString } from '../helpers/split-string.js';
import { toDateString } from '../helpers/to-date-string.js';
import { toFormatters } from '../helpers/to-formatters.js';
import { toResolvedDate } from '../helpers/to-resolved-date.js';
import { appIconButtonName } from '../icon-button/constants.js';
import { iconArrowDropdown, iconChevronLeft, iconChevronRight } from '../icons.js';
import { DatePickerMinMaxMixin } from '../mixins/date-picker-min-max-mixin.js';
import { DatePickerMixin } from '../mixins/date-picker-mixin.js';
import { RootElement } from '../root-element/root-element.js';
import { baseStyling, resetShadowRoot, webkitScrollbarStyling } from '../stylings.js';
import { datePickerStyling } from './stylings.js';
export class DatePicker extends DatePickerMixin(DatePickerMinMaxMixin(RootElement)) {
    constructor() {
        super();
        _DatePicker_formatters.set(this, void 0);
        _DatePicker_focusNavButtonWithKey.set(this, false);
        _DatePicker_today.set(this, void 0);
        _DatePicker_valueAsDate.set(this, void 0);
        _DatePicker_navigateMonth.set(this, (ev) => {
            const currentDate = this._currentDate;
            const isPreviousNavigation = ev.currentTarget.getAttribute('data-navigation') === 'previous';
            const newCurrentDate = toUTCDate(currentDate.getUTCFullYear(), currentDate.getUTCMonth() + (isPreviousNavigation ? -1 : 1), 1);
            this._currentDate = newCurrentDate;
            __classPrivateFieldSet(this, _DatePicker_focusNavButtonWithKey, ev.detail === 0, "f");
        });
        _DatePicker_queryAllFocusable.set(this, async () => {
            const isStartViewCalendar = this.startView === 'calendar';
            const focusable = [
                ...this.queryAll(appIconButtonName),
                (await (isStartViewCalendar ? this._monthCalendar : this._yearGrid))
                    ?.query(`.${isStartViewCalendar ? 'calendar-day' : 'year-grid-button'}[aria-selected="true"]`),
            ].filter(Boolean);
            return focusable;
        });
        _DatePicker_renderCalendar.set(this, () => {
            const { _currentDate, _max, _min, _selectedDate, disabledDates, disabledDays, firstDayOfWeek, locale, selectedDateLabel, shortWeekLabel, showWeekNumber, todayLabel, weekLabel, weekNumberTemplate, weekNumberType, } = this;
            const currentDate = _currentDate;
            const formatters = __classPrivateFieldGet(this, _DatePicker_formatters, "f");
            const max = _max;
            const min = _min;
            const selectedDate = _selectedDate;
            const { dayFormat, fullDateFormat, longWeekdayFormat, narrowWeekdayFormat, } = __classPrivateFieldGet(this, _DatePicker_formatters, "f");
            const weekdays = getWeekdays({
                firstDayOfWeek,
                longWeekdayFormat,
                narrowWeekdayFormat,
                shortWeekLabel,
                showWeekNumber,
                weekLabel,
            });
            const { calendar: calendarMonth, disabledDatesSet, disabledDaysSet, } = calendar({
                date: currentDate,
                dayFormat,
                disabledDates: splitString(disabledDates, toResolvedDate),
                disabledDays: splitString(disabledDays, Number),
                firstDayOfWeek,
                fullDateFormat,
                locale,
                max,
                min,
                showWeekNumber,
                weekNumberTemplate,
                weekNumberType,
            });
            return html `
    <app-month-calendar
      .data=${{
                calendar: calendarMonth,
                currentDate,
                date: selectedDate,
                disabledDatesSet,
                disabledDaysSet,
                formatters,
                max,
                min,
                selectedDateLabel,
                showWeekNumber,
                todayDate: __classPrivateFieldGet(this, _DatePicker_today, "f"),
                todayLabel,
                weekdays,
            }}
      @date-updated=${__classPrivateFieldGet(this, _DatePicker_updateSelectedDate, "f")}
      class=calendar
      exportparts=table,caption,weekdays,weekday,weekday-value,week-number,calendar-day,today,calendar
    ></app-month-calendar>
    `;
        });
        _DatePicker_renderNavigationButton.set(this, (navigationType, shouldSkipRender = true) => {
            const isPreviousNavigationType = navigationType === 'previous';
            const label = isPreviousNavigationType ? this.previousMonthLabel : this.nextMonthLabel;
            return shouldSkipRender ?
                html `<div data-navigation=${navigationType}></div>` :
                html `
      <app-icon-button
        .ariaLabel=${label}
        @click=${__classPrivateFieldGet(this, _DatePicker_navigateMonth, "f")}
        data-navigation=${navigationType}
        title=${ifDefined(label)}
      >${isPreviousNavigationType ? iconChevronLeft : iconChevronRight}</app-icon-button>
      `;
        });
        _DatePicker_renderYearGrid.set(this, () => {
            const { _max, _min, _selectedDate, selectedYearLabel, toyearLabel, } = this;
            return html `
    <app-year-grid
      class=year-grid
      .data=${{
                date: _selectedDate,
                formatters: __classPrivateFieldGet(this, _DatePicker_formatters, "f"),
                max: _max,
                min: _min,
                selectedYearLabel,
                toyearLabel,
            }}
      @year-updated=${__classPrivateFieldGet(this, _DatePicker_updateYear, "f")}
      exportparts=year-grid,year,toyear
    ></app-year-grid>
    `;
        });
        _DatePicker_updateSelectedDate.set(this, ({ detail: { value }, }) => {
            const selectedDate = new Date(value);
            this._selectedDate = selectedDate;
            this._currentDate = new Date(selectedDate);
            this.value = toDateString(selectedDate);
        });
        _DatePicker_updateStartView.set(this, () => {
            this.startView = this.startView === 'yearGrid' ? 'calendar' : 'yearGrid';
        });
        _DatePicker_updateYear.set(this, ({ detail: { year }, }) => {
            this._currentDate = new Date(this._currentDate.setUTCFullYear(year));
            this.startView = 'calendar';
        });
        const todayDate = toResolvedDate();
        this._min = new Date(todayDate);
        this._max = new Date(MAX_DATE);
        __classPrivateFieldSet(this, _DatePicker_today, todayDate, "f");
        this._selectedDate = new Date(todayDate);
        this._currentDate = new Date(todayDate);
        __classPrivateFieldSet(this, _DatePicker_formatters, toFormatters(this.locale), "f");
        __classPrivateFieldSet(this, _DatePicker_valueAsDate, new Date(todayDate), "f");
    }
    get valueAsDate() {
        return __classPrivateFieldGet(this, _DatePicker_valueAsDate, "f");
    }
    get valueAsNumber() {
        return +__classPrivateFieldGet(this, _DatePicker_valueAsDate, "f");
    }
    willUpdate(changedProperties) {
        if (changedProperties.has('locale')) {
            const newLocale = (this.locale || DateTimeFormat().resolvedOptions().locale);
            __classPrivateFieldSet(this, _DatePicker_formatters, toFormatters(newLocale), "f");
            this.locale = newLocale;
        }
        const dateRangeProps = [
            'max',
            'min',
            'value',
        ];
        if (dateRangeProps.some(n => changedProperties.has(n))) {
            const todayDate = __classPrivateFieldGet(this, _DatePicker_today, "f");
            const [newMax, newMin, newValue,] = [
                ['max', MAX_DATE],
                ['min', todayDate],
                ['value', todayDate],
            ].map(([propKey, resetValue]) => {
                const currentValue = this[propKey];
                const defaultValue = toResolvedDate(changedProperties.get(propKey) ?? resetValue);
                const valueWithReset = currentValue === undefined ? resetValue : currentValue;
                return dateValidator(valueWithReset, defaultValue);
            });
            const valueDate = toResolvedDate(clampValue(+newMin.date, +newMax.date, +newValue.date));
            this._min = newMin.date;
            this._max = newMax.date;
            this._currentDate = new Date(valueDate);
            this._selectedDate = new Date(valueDate);
            __classPrivateFieldSet(this, _DatePicker_valueAsDate, new Date(valueDate), "f");
            if (!this.max)
                this.max = toDateString(newMax.date);
            if (!this.min)
                this.min = toDateString(newMin.date);
            if (!this.value) {
                const valueStr = toDateString(valueDate);
                this.value = valueStr;
                this.fire({
                    detail: {
                        isKeypress: false,
                        value: valueStr,
                        valueAsDate: new Date(valueDate),
                        valueAsNumber: +valueDate,
                    },
                    type: 'date-updated',
                });
            }
        }
        if (changedProperties.has('startView')) {
            const oldStartView = (changedProperties.get('startView') || 'calendar');
            const { _max, _min, _selectedDate, startView, } = this;
            if (!startViews.includes(startView)) {
                this.startView = oldStartView;
            }
            if (startView === 'calendar') {
                const newSelectedYear = new Date(clampValue(+_min, +_max, +_selectedDate));
                this._selectedDate = newSelectedYear;
            }
        }
    }
    async firstUpdated() {
        const valueAsDate = __classPrivateFieldGet(this, _DatePicker_valueAsDate, "f");
        this.fire({
            detail: {
                focusableElements: await __classPrivateFieldGet(this, _DatePicker_queryAllFocusable, "f").call(this),
                value: toDateString(valueAsDate),
                valueAsDate: new Date(valueAsDate),
                valueAsNumber: +valueAsDate,
            },
            type: 'first-updated',
        });
    }
    async updated(changedProperties) {
        const { _currentDate, _max, _min, _navigationNext, _navigationPrevious, _yearDropdown, startView, } = this;
        if (changedProperties.get('startView') === 'yearGrid' &&
            startView === 'calendar') {
            (await _yearDropdown)?.focus();
        }
        if (startView === 'calendar') {
            if (changedProperties.has('_currentDate') && __classPrivateFieldGet(this, _DatePicker_focusNavButtonWithKey, "f")) {
                isInCurrentMonth(_min, _currentDate) && focusElement(_navigationNext);
                isInCurrentMonth(_max, _currentDate) && focusElement(_navigationPrevious);
                __classPrivateFieldSet(this, _DatePicker_focusNavButtonWithKey, false, "f");
            }
        }
    }
    render() {
        const { _currentDate, _max, _min, chooseMonthLabel, chooseYearLabel, showWeekNumber, startView, } = this;
        const formatters = __classPrivateFieldGet(this, _DatePicker_formatters, "f");
        const { longMonthYearFormat } = formatters;
        const selectedYearMonth = longMonthYearFormat(_currentDate);
        const isStartViewYearGrid = startView === 'yearGrid';
        const label = isStartViewYearGrid ? chooseMonthLabel : chooseYearLabel;
        return html `
    <div class=header part=header>
      <div class=month-and-year-selector>
        <p class=selected-year-month>${selectedYearMonth}</p>

        <app-icon-button
          .ariaLabel=${label}
          @click=${__classPrivateFieldGet(this, _DatePicker_updateStartView, "f")}
          class=year-dropdown
          title=${ifDefined(label)}
        >${iconArrowDropdown}</app-icon-button>
      </div>

      ${isStartViewYearGrid ?
            nothing :
            html `
          <div class=month-pagination>
            ${__classPrivateFieldGet(this, _DatePicker_renderNavigationButton, "f").call(this, 'previous', isInCurrentMonth(_min, _currentDate))}
            ${__classPrivateFieldGet(this, _DatePicker_renderNavigationButton, "f").call(this, 'next', isInCurrentMonth(_max, _currentDate))}
          </div>
          `}
    </div>

    <div class="body ${classMap({
            [`start-view--${startView}`]: true,
            'show-week-number': showWeekNumber,
        })}" part=body>${(isStartViewYearGrid ? __classPrivateFieldGet(this, _DatePicker_renderYearGrid, "f") : __classPrivateFieldGet(this, _DatePicker_renderCalendar, "f"))()}</div>
    `;
    }
}
_DatePicker_formatters = new WeakMap(), _DatePicker_focusNavButtonWithKey = new WeakMap(), _DatePicker_today = new WeakMap(), _DatePicker_valueAsDate = new WeakMap(), _DatePicker_navigateMonth = new WeakMap(), _DatePicker_queryAllFocusable = new WeakMap(), _DatePicker_renderCalendar = new WeakMap(), _DatePicker_renderNavigationButton = new WeakMap(), _DatePicker_renderYearGrid = new WeakMap(), _DatePicker_updateSelectedDate = new WeakMap(), _DatePicker_updateStartView = new WeakMap(), _DatePicker_updateYear = new WeakMap();
DatePicker.styles = [
    baseStyling,
    resetShadowRoot,
    datePickerStyling,
    webkitScrollbarStyling,
];
__decorate([
    queryAsync('app-month-calendar')
], DatePicker.prototype, "_monthCalendar", void 0);
__decorate([
    queryAsync('[data-navigation="previous"]')
], DatePicker.prototype, "_navigationPrevious", void 0);
__decorate([
    queryAsync('[data-navigation="next"]')
], DatePicker.prototype, "_navigationNext", void 0);
__decorate([
    queryAsync('.year-dropdown')
], DatePicker.prototype, "_yearDropdown", void 0);
__decorate([
    queryAsync('app-year-grid')
], DatePicker.prototype, "_yearGrid", void 0);
__decorate([
    state()
], DatePicker.prototype, "_currentDate", void 0);
__decorate([
    state()
], DatePicker.prototype, "_max", void 0);
__decorate([
    state()
], DatePicker.prototype, "_min", void 0);
__decorate([
    state()
], DatePicker.prototype, "_selectedDate", void 0);
